@@ -29,7 +29,11 @@ void FastText::getVector(Vector& vec, const std::string& word) const {
   const std::vector<int32_t>& ngrams = dict_->getSubwords(word);
   vec.zero();
   for (auto it = ngrams.begin(); it != ngrams.end(); ++it) {
-    vec.addRow(*input_, *it);
+    if (quant_) {
+      vec.addRow(*qinput_, *it);
+    } else {
+      vec.addRow(*input_, *it);
+    }
   }
   if (ngrams.size() > 0) {
     vec.mul(1.0 / ngrams.size());
@@ -58,10 +62,18 @@ void FastText::saveOutput() {
     std::cerr << "Error opening file for saving vectors." << std::endl;
     exit(EXIT_FAILURE);
   }
-  ofs << dict_->nwords() << " " << args_->dim << std::endl;
+  if (quant_) {
+    std::cerr << "Option -saveOutput is not supported for quantized models."
+              << std::endl;
+    return;
+  }
+  int32_t n = (args_->model == model_name::sup) ? dict_->nlabels()
+                                                : dict_->nwords();
+  ofs << n << " " << args_->dim << std::endl;
   Vector vec(args_->dim);
-  for (int32_t i = 0; i < dict_->nwords(); i++) {
-    std::string word = dict_->getWord(i);
+  for (int32_t i = 0; i < n; i++) {
+    std::string word = (args_->model == model_name::sup) ? dict_->getLabel(i)
+                                                         : dict_->getWord(i);
     vec.zero();
     vec.addRow(*output_, i);
     ofs << word << " " << vec << std::endl;
@@ -71,13 +83,12 @@ void FastText::saveOutput() {
 
 bool FastText::checkModel(std::istream& in) {
   int32_t magic;
-  int32_t version;
   in.read((char*)&(magic), sizeof(int32_t));
   if (magic != FASTTEXT_FILEFORMAT_MAGIC_INT32) {
     return false;
   }
   in.read((char*)&(version), sizeof(int32_t));
-  if (version != FASTTEXT_VERSION) {
+  if (version > FASTTEXT_VERSION) {
     return false;
   }
   return true;
@@ -145,7 +156,10 @@ void FastText::loadModel(std::istream& in) {
   qinput_ = std::make_shared<QMatrix>();
   qoutput_ = std::make_shared<QMatrix>();
   args_->load(in);
-
+  if (version == 11 && args_->model == model_name::sup) {
+    // backward compatibility: old supervised models do not use char ngrams.
+    args_->maxn = 0;
+  }
   dict_->load(in);
 
   bool quant_input;
@@ -157,6 +171,13 @@ void FastText::loadModel(std::istream& in) {
     input_->load(in);
   }
 
+  if (!quant_input && dict_->isPruned()) {
+    std::cerr << "Invalid model file.\n"
+              << "Please download the updated model from www.fasttext.cc.\n"
+              << "See issue #332 on Github for more information.\n";
+    exit(1);
+  }
+
   in.read((char*) &args_->qout, sizeof(bool));
   if (quant_ && args_->qout) {
     qoutput_->load(in);
@@ -207,7 +228,8 @@ std::vector<int32_t> FastText::selectEmbeddings(int32_t cutoff) const {
 
 void FastText::quantize(std::shared_ptr<Args> qargs) {
   if (qargs->output.empty()) {
-      std::cerr<<"No model provided!"<<std::endl; exit(1);
+    std::cerr<<"No model provided!"<<std::endl;
+    exit(1);
   }
   loadModel(qargs->output + ".bin");
 
@@ -234,6 +256,7 @@ void FastText::quantize(std::shared_ptr<Args> qargs) {
       args_->verbose = qargs->verbose;
       start = clock();
       tokenCount = 0;
+      start = clock();
       std::vector<std::thread> threads;
       for (int32_t i = 0; i < args_->thread; i++) {
         threads.push_back(std::thread([=]() { trainThread(i); }));
@@ -325,6 +348,7 @@ void FastText::predict(std::istream& in, int32_t k,
   std::vector<int32_t> words, labels;
   predictions.clear();
   dict_->getLine(in, words, labels, model_->rng);
+  predictions.clear();
   if (words.empty()) return;
   Vector hidden(args_->dim);
   Vector output(dict_->nlabels());
@@ -338,6 +362,7 @@ void FastText::predict(std::istream& in, int32_t k,
 void FastText::predict(std::istream& in, int32_t k, bool print_prob) {
   std::vector<std::pair<real,std::string>> predictions;
   while (in.peek() != EOF) {
+    predictions.clear();
     predict(in, k, predictions);
     if (predictions.empty()) {
       std::cout << std::endl;
@@ -390,6 +415,10 @@ void FastText::sentenceVectors() {
   }
 }
 
+std::shared_ptr<const Dictionary> FastText::getDictionary() const {
+  return dict_;
+}
+
 void FastText::ngramVectors(std::string word) {
   std::vector<int32_t> ngrams;
   std::vector<std::string> substrings;
@@ -398,7 +427,11 @@ void FastText::ngramVectors(std::string word) {
   for (int32_t i = 0; i < ngrams.size(); i++) {
     vec.zero();
     if (ngrams[i] >= 0) {
-      vec.addRow(*input_, ngrams[i]);
+      if (quant_) {
+        vec.addRow(*qinput_, ngrams[i]);
+      } else {
+        vec.addRow(*input_, ngrams[i]);
+      }
     }
     std::cout << substrings[i] << " " << vec << std::endl;
   }
@@ -411,7 +444,11 @@ void FastText::textVectors() {
     dict_->getLine(std::cin, line, labels, model_->rng);
     vec.zero();
     for (auto it = line.cbegin(); it != line.cend(); ++it) {
-      vec.addRow(*input_, *it);
+      if (quant_) {
+        vec.addRow(*qinput_, *it);
+      } else {
+        vec.addRow(*input_, *it);
+      }
     }
     if (!line.empty()) {
       vec.mul(1.0 / line.size());
@@ -532,12 +569,14 @@ void FastText::trainThread(int32_t threadId) {
   while (tokenCount < args_->epoch * ntokens) {
     real progress = real(tokenCount) / (args_->epoch * ntokens);
     real lr = args_->lr * (1.0 - progress);
-    localTokenCount += dict_->getLine(ifs, line, labels, model.rng);
     if (args_->model == model_name::sup) {
+      localTokenCount += dict_->getLine(ifs, line, labels, model.rng);
       supervised(model, lr, line, labels);
     } else if (args_->model == model_name::cbow) {
+      localTokenCount += dict_->getLine(ifs, line, model.rng);
       cbow(model, lr, line);
     } else if (args_->model == model_name::sg) {
+      localTokenCount += dict_->getLine(ifs, line, model.rng);
       skipgram(model, lr, line);
     }
     if (localTokenCount > args_->lrUpdateRate) {
@@ -641,11 +680,9 @@ void FastText::train(std::shared_ptr<Args> args) {
   model_ = std::make_shared<Model>(input_, output_, args_, 0);
 
   saveModel();
-  if (args_->model != model_name::sup) {
-    saveVectors();
-    if (args_->saveOutput > 0) {
-      saveOutput();
-    }
+  saveVectors();
+  if (args_->saveOutput > 0) {
+    saveOutput();
   }
 }
 
